@@ -694,6 +694,8 @@ MixfixModule::newFancySymbol(Token prefixName,
       return new FileManagerSymbol(name);
     case SymbolType::STREAM_MANAGER_SYMBOL:
       return new StreamManagerSymbol(name);
+    case SymbolType::PROCESS_MANAGER_SYMBOL:
+      return new ProcessManagerSymbol(name);
     }
 
   int lineNr = prefixName.lineNumber();
